@@ -42,7 +42,6 @@ from ..jsonutil import (
     JsonError,
     JsonObject,
     get_bool,
-    get_enum,
     get_int,
     get_str,
     get_strv,
@@ -58,11 +57,11 @@ def my_umask() -> int:
     return (match and int(match.group(1), 8)) or 0o077
 
 
-def tag_from_stat(buf):
+def tag_from_stat(buf) -> str:
     return f'1:{buf.st_ino}-{buf.st_mtime}-{buf.st_mode:o}-{buf.st_uid}-{buf.st_gid}'
 
 
-def tag_from_path(path):
+def tag_from_path(path) -> 'str | None':
     try:
         return tag_from_stat(os.stat(path))
     except FileNotFoundError:
@@ -71,13 +70,15 @@ def tag_from_path(path):
         return None
 
 
-def tag_from_fd(fd):
+def tag_from_fd(fd) -> 'str | None':
     try:
         return tag_from_stat(os.fstat(fd))
     except OSError:
         return None
 
 
+# DEPRECATED: https://github.com/cockpit-project/cockpit/pull/21055
+# as of 2024-09-30 there are no more users of this in any known project
 class FsListChannel(Channel):
     payload = 'fslist1'
 
@@ -123,7 +124,6 @@ class FsReadChannel(GeneratorChannel):
 
     def do_yield_data(self, options: JsonObject) -> Generator[bytes, None, JsonObject]:
         path = get_str(options, 'path')
-        binary = get_enum(options, 'binary', ['raw'], None) is not None
         max_read_size = get_int(options, 'max_read_size', None)
 
         logger.debug('Opening file "%s" for reading', path)
@@ -134,7 +134,7 @@ class FsReadChannel(GeneratorChannel):
                 if max_read_size is not None and buf.st_size > max_read_size:
                     raise ChannelError('too-large')
 
-                if binary and stat.S_ISREG(buf.st_mode):
+                if self.is_binary and stat.S_ISREG(buf.st_mode):
                     self.ready(size_hint=buf.st_size)
                 else:
                     self.ready()
@@ -144,14 +144,15 @@ class FsReadChannel(GeneratorChannel):
                     if data == b'':
                         break
                     logger.debug('  ...sending %d bytes', len(data))
-                    if not binary:
-                        data = data.replace(b'\0', b'').decode('utf-8', errors='ignore').encode('utf-8')
+                    if not self.is_binary:
+                        data = data.replace(b'\0', b'').decode(errors='ignore').encode()
                     yield data
 
             return {'tag': tag_from_stat(buf)}
 
         except FileNotFoundError:
-            return {'tag': '-'}
+            # Using `yield` and `return {value}` generator, but GeneratorChannel does expect this
+            return {'tag': '-'}  # noqa: B901
         except PermissionError as exc:
             raise ChannelError('access-denied') from exc
         except OSError as exc:
@@ -168,7 +169,9 @@ class FsReplaceChannel(AsyncChannel):
             os.unlink(path)
         return '-'
 
-    async def set_contents(self, path: str, tag: 'str | None', data: 'bytes | None', size: 'int | None') -> str:
+    async def set_contents(
+        self, path: str, tag: 'str | None', data: 'bytes | None', size: 'int | None'
+    ) -> 'str | None':
         dirname, basename = os.path.split(path)
         tmpname: str | None
         fd, tmpname = tempfile.mkstemp(dir=dirname, prefix=f'.{basename}-')
@@ -259,10 +262,9 @@ class FsReplaceChannel(AsyncChannel):
             raise ChannelError('internal-error', message=str(exc)) from exc
 
 
-class FsWatchChannel(Channel):
+class FsWatchChannel(Channel, PathWatchListener):
     payload = 'fswatch1'
-    _tag = None
-    _path = None
+    _tag: 'str | None' = None
     _watch = None
 
     # The C bridge doesn't send the initial event, and the JS calls read()
@@ -272,7 +274,7 @@ class FsWatchChannel(Channel):
     _active = False
 
     @staticmethod
-    def mask_to_event_and_type(mask):
+    def mask_to_event_and_type(mask: InotifyEvent) -> 'tuple[str, str | None]':
         if (InotifyEvent.CREATE or InotifyEvent.MOVED_TO) in mask:
             return 'created', 'directory' if InotifyEvent.ISDIR in mask else 'file'
         elif InotifyEvent.MOVED_FROM in mask or InotifyEvent.DELETE in mask or InotifyEvent.DELETE_SELF in mask:
@@ -284,7 +286,7 @@ class FsWatchChannel(Channel):
         else:
             return 'changed', None
 
-    def do_inotify_event(self, mask, _cookie, name):
+    def do_inotify_event(self, mask: InotifyEvent, _cookie: int, name: 'bytes | None') -> None:
         logger.debug("do_inotify_event(%s): mask %X name %s", self._path, mask, name)
         event, type_ = self.mask_to_event_and_type(mask)
         if name:
@@ -300,14 +302,14 @@ class FsWatchChannel(Channel):
             self._tag = tag
             self.send_json(event=event, path=self._path, tag=self._tag, type=type_)
 
-    def do_identity_changed(self, fd, err):
+    def do_identity_changed(self, fd: 'int | None', err: 'int | None') -> None:
         logger.debug("do_identity_changed(%s): fd %s, err %s", self._path, str(fd), err)
         self._tag = tag_from_fd(fd) if fd else '-'
         if self._active:
             self.send_json(event='created' if fd else 'deleted', path=self._path, tag=self._tag)
 
-    def do_open(self, options):
-        self._path = options['path']
+    def do_open(self, options: JsonObject) -> None:
+        self._path = get_str(options, 'path')
         self._tag = None
 
         self._active = False
@@ -316,9 +318,10 @@ class FsWatchChannel(Channel):
 
         self.ready()
 
-    def do_close(self):
-        self._watch.close()
-        self._watch = None
+    def do_close(self) -> None:
+        if self._watch is not None:
+            self._watch.close()
+            self._watch = None
         self.close()
 
 
